@@ -8,19 +8,34 @@
  work, as well as functions which operate on them.
 """
 from types import FunctionType as Function
-from typing import Any, Dict, Generic, Hashable, List, Type, TypeVar, Union
+from typing import Any, Dict, Hashable, List, Type, TypeVar, Union
 
 DataClass = Any  # type hint for variables that should be data class instances
-T = TypeVar('T')
 
 
-class Internal(Generic[T]):
-    """This type hint wrapper represents a field that is internal to the data class and is, for example, not to be
-    shown in a repr."""
+class Hint:
+    """A type hint "wrapper". Wraps the actual type of a field to convey information about how it is intended
+    to be used, much like typing.ClassVar. Usage is like Hint[some_type]."""
+    Wrapped = TypeVar('Wrapped')
+
+    def __class_getitem__(cls, item: Wrapped) -> Union['Hint', Wrapped]:
+        """Create a new Union of the wrapper and the wrapped type. Union is smart enough to flatten nested
+        unions automatically."""
+        return Union[cls, item]
+
     @classmethod
-    def is_internal(cls, annotation: Union[Type, str]):
-        return getattr(annotation, '__origin__', None) is cls or \
-               (type(annotation) is str and annotation.startswith(f'{cls.__name__}['))
+    def is_hinted(cls, hint: Union[Type, str]) -> bool:
+        """Check whether a type hint represents Internal."""
+        return (hasattr(hint, '__args__') and cls in hint.__args__ or
+                (type(hint) is str and f'{cls.__name__}[' in hint))
+
+
+class Internal(Hint):
+    """Marks that a field is internal to the class and so should not in a repr."""
+
+
+class Hashed(Hint):
+    """Marks that a field should be included in the generated __hash__."""
 
 
 class DataClassMeta(type):
@@ -34,19 +49,17 @@ class DataClassMeta(type):
         all_defaults = {}
         all_slots = set()
         options = dict(mcs.DEFAULT_OPTIONS)
+        post_init = False  # whether a post-initialisation method was defined by the user
 
         dataclass_bases = [vars(b) for b in bases if hasattr(b, '__dataclass__')]
         for b in dataclass_bases + [dict_]:
             all_annotations.update(b.get('__annotations__', {}))
-            all_defaults.update(b.get('__defaults__', dict_))
+            all_defaults.update({f: v for f, v in b.get('__defaults__', dict_).items() if f in all_annotations})
             all_slots.update(b.get('__slots__', set()))
             options.update(b.get('__dataclass__', {}))
-        options.update(kwargs)
 
-        # determine whether a post-initialisation method was defined by the user
-        init = dict_.get('__init__')
-        post_init = (type(init) is Function and not hasattr(init, '__dataclass__')
-                     or type(dict_.get('__post_init__')) is Function)
+            post_init = post_init or is_user_func(b.get('__init__')) or is_user_func(b.get('__post_init__'))
+        options.update(kwargs)
 
         # fill out this class' dict and store defaults, annotations and decorator options for future subclasses
         dict_.update(all_defaults)
@@ -55,13 +68,14 @@ class DataClassMeta(type):
         dict_['__dataclass__'] = options
 
         # delete what will become stale references so that Python creates new ones
-        del dict_['__dict__'], dict_['__weakref__']
+        dict_.pop('__dict__', None)
+        dict_.pop('__weakref__', None)
 
         # create/apply generated methods and attributes
         if options['slots']:
             # if the slots option is added, specify them. Values with default values must only be present in slots,
             # not dict, otherwise Python will interpret them as read only
-            for d in all_annotations.keys() & all_defaults.keys():
+            for d in all_defaults.keys():
                 del dict_[d]
             dict_['__slots__'] = all_annotations.keys() - all_slots
         elif '__slots__' in dict_:
@@ -94,7 +108,7 @@ class DataClassMeta(type):
         if (options['eq'] and options['frozen']) or options['unsafe_hash']:
             dict_.setdefault('__hash__', __hash__)
 
-        return type.__new__(mcs, name, bases, dict_)
+        return super().__new__(mcs, name, bases, dict_)
 
     # noinspection PyMissingConstructor,PyUnresolvedReferences,PyTypeChecker,PyUnusedLocal
     def __init__(cls, *args, **kwargs):
@@ -148,6 +162,12 @@ def eval_function(name: str, lines: List[str], annotations: Dict, locals_: Dict,
     return function
 
 
+def is_user_func(obj: Any) -> bool:
+    """Returns whether an object is a function explicitly defined by the user (i.e. not generated by
+    dataclassy and not a wrapper/descriptor)."""
+    return type(obj) is Function and not hasattr(obj, '__dataclass__')
+
+
 # generic method implementations common to all data classes
 from .functions import values, fields
 
